﻿//##################################################################################################
//
//  Ceetron Export
//  VTFxExample
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Ceetron.Ceetron3DComponents;
using Ceetron.Ceetron3DComponents.VTFx;

namespace Example1
{
    class Example1
    {
        private bool allAtOnce = true;

        // Part 1
        // -----------------------------------------------------------------------------
        // Nodes for part 1
        static float[] NODES_PART_1 = new float[]
        {
            0.0f, 0.0f, 0.0f,
            1.0f, 0.0f, 0.0f,
            1.0f, 1.0f, 0.0f,
            0.0f, 1.0f, 0.0f,
            0.0f, 0.0f, 1.0f,
            1.0f, 0.0f, 1.0f,
            1.0f, 1.0f, 1.0f,
            0.0f, 1.0f, 1.0f,
            0.0f, 0.0f, 2.0f,
            1.0f, 0.0f, 2.0f,
            1.0f, 1.0f, 2.0f,
            0.0f, 1.0f, 2.0f
        };

        static int[] NODES_PART_1_IDS = new int[]
        {
            1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12
        };


        // Elements for part 1
        static int[] CONNECTS_PART_1 = new int[]
        {
            0, 1, 2, 3, 4, 5, 6, 7,
            4, 5, 6, 7, 8, 9, 10, 11
        };
        ElementBlock.ElementType elementTypePart1 = ElementBlock.ElementType.HEXAHEDRONS;

        static int[] ELEMENTS_PART_1_IDS = new int[]
        {
            1, 2
        };


        // Part 2
        // -----------------------------------------------------------------------------
        // Nodes for part 2
        static float[] NODES_PART_2 = new float[]
        {
            2.0f, 0.0f, 0.0f,
            3.0f, 0.0f, 0.0f,
            3.0f, 1.0f, 0.0f,
            2.0f, 1.0f, 0.0f,
            4.0f, 0.0f, 0.0f,
            4.0f, 1.0f, 0.0f
        };

        static int[] NODES_PART_2_IDS = new int[]
        {
            101, 102, 103, 104, 105, 106
        };


        // Elements for part 2
        static int[] CONNECTS_PART_2 = new int[]
        {
            0, 1, 2, 3,
            1, 4, 5, 2
        };
        ElementBlock.ElementType elementTypePart2 = ElementBlock.ElementType.QUADS;

        static int[] ELEMENTS_PART_2_IDS = new int[]
        {
            3, 4
        };


        // Geometry definition
        // -----------------------------------------------------------------------------
        static int[] ELEMENT_PARTS = new int[]
        {
            1, 2
        };
        

        // Time step 1 - Scalar Result 1
        // -----------------------------------------------------------------------------
        static float[] RESULT1_P1_S1 = new float[]
        {
            0.0f,
            0.5f,
            1.5f,
            2.0f,
            3.0f,
            4.0f,
            5.0f,
            6.0f,
            6.5f,
            7.0f,
            7.5f,
            8.0f
        };
        static float[] RESULT1_P2_S1 = new float[]
        {
            1.0f,
            5.0f,
            3.0f,
            2.0f,
            5.0f,
            1.0f
        };


        // Time step 2 - Scalar Result 1
        // -----------------------------------------------------------------------------
        static float[] RESULT1_P1_S2 = new float[]
        {
            8.1f,
            9.6f,
            5.8f,
            3.5f,
            7.4f,
            9.7f,
            2.7f,
            6.5f,
            3.2f,
            4.6f,
            3.7f,
            2.2f
        };
        static float[] RESULT1_P2_S2 = new float[]
        {
            1.5f,
            2.1f,
            4.5f,
            5.5f,
            2.1f,
            1.1f
        };


        // Time step 1 - Vector Result 1
        // -----------------------------------------------------------------------------
        static float[] RESULT2_P1_S1 = new float[]
        {
            0.0f, 0.0f, 0.0f,
            1.0f, 0.0f, 0.0f,
            1.0f, 1.0f, 0.0f,
            0.0f, 1.0f, 0.0f,
            0.0f, 0.0f, 1.0f,
            1.0f, 0.0f, 1.0f,
            1.0f, 1.0f, 1.0f,
            0.0f, 1.0f, 1.0f,
            0.0f, 0.0f, 2.0f,
            1.0f, 0.0f, 2.0f,
            1.0f, 1.0f, 2.0f,
            0.0f, 1.0f, 2.0f
        };
        static float[] RESULT2_P2_S1 = new float[]
        {
            2.0f, 0.0f, 0.0f,
            3.0f, 0.0f, 0.0f,
            3.0f, 1.0f, 0.0f,
            2.0f, 1.0f, 0.0f,
            4.0f, 0.0f, 0.0f,
            4.0f, 1.0f, 0.0f,
        };


        // Time step 2 - Vector Result 2
        // -----------------------------------------------------------------------------
        static float[] RESULT2_P1_S2 = new float[]
        {
            1.0f, 8.0f, 1.0f,
            3.0f, 2.0f, 5.0f,
            5.0f, 8.0f, 7.0f,
            3.0f, 4.0f, 9.0f,
            5.0f, 2.0f, 3.0f,
            7.0f, 1.0f, 2.0f,
            2.0f, 7.0f, 1.0f,
            5.0f, 6.0f, 5.0f,
            7.0f, 5.0f, 8.0f,
            6.0f, 4.0f, 0.0f,
            4.0f, 2.0f, 0.0f,
            1.0f, 2.0f, 2.0f
        };
        static float[] RESULT2_P2_S2 = new float[]
        {
            1.0f, 4.0f, 2.0f,
            6.0f, 8.0f, 5.0f,
            8.0f, 7.0f, 7.0f,
            3.0f, 3.0f, 8.0f,
            7.0f, 5.0f, 2.0f,
            3.0f, 3.0f, 1.0f,
        };


        // Time step 1 - Displacement Result 1
        // -----------------------------------------------------------------------------
        static float[] RESULT3_P1_S1 = new float[]
        {
            0.0f, 0.0f, 0.0f,
            1.0f, 0.0f, 0.0f,
            1.0f, 1.0f, 0.0f,
            0.0f, 1.0f, 0.0f,
            0.0f, 0.0f, 1.0f,
            1.0f, 0.0f, 1.0f,
            1.0f, 1.0f, 1.0f,
            0.0f, 1.0f, 1.0f,
            0.0f, 0.0f, 2.0f,
            1.0f, 0.0f, 2.0f,
            1.0f, 1.0f, 2.0f,
            0.0f, 1.0f, 2.0f
        };
        static float[] RESULT3_P2_S1 = new float[]
        {
            2.0f, 0.0f, 0.0f,
            3.0f, 0.0f, 0.0f,
            3.0f, 1.0f, 0.0f,
            2.0f, 1.0f, 0.0f,
            4.0f, 0.0f, 0.0f,
            4.0f, 1.0f, 0.0f,
        };


        // Time step 2 - Displacement Result 1
        // -----------------------------------------------------------------------------
        static float[] RESULT3_P1_S2 = new float[]
        {
            1.0f, 0.0f, 0.0f,
            2.0f, 0.0f, 0.0f,
            2.0f, 1.0f, 0.0f,
            1.0f, 1.0f, 0.0f,
            1.0f, 0.0f, 1.0f,
            2.0f, 0.0f, 1.0f,
            2.0f, 1.0f, 1.0f,
            1.0f, 1.0f, 1.0f,
            1.0f, 0.0f, 2.0f,
            2.0f, 0.0f, 2.0f,
            2.0f, 1.0f, 2.0f,
            1.0f, 1.0f, 2.0f
        };
        static float[] RESULT3_P2_S2 = new float[]
        {
            3.0f, 0.0f, 0.0f,
            4.0f, 0.0f, 0.0f,
            4.0f, 1.0f, 0.0f,
            3.0f, 1.0f, 0.0f,
            5.0f, 0.0f, 0.0f,
            5.0f, 1.0f, 0.0f,
        };


        //--------------------------------------------------------------------------------------------------
        // Constructor
        //--------------------------------------------------------------------------------------------------
        public Example1()
        {
        }


        //--------------------------------------------------------------------------------------------------
        // Create the VTFx file
        //--------------------------------------------------------------------------------------------------
        public bool createModelFile(string filename)
        {
            // Create the VTFx file
            // -------------------------------------------------------------------------
            File file = new File();

            // Create VTFx file settings struct
            // -------------------------------------------------------------------------
            FileSettings fileSettings = new FileSettings();
            fileSettings.Binary = true;
            fileSettings.ApplicationName = "Ceetron Export: .NET Example";
            fileSettings.VendorName = "My Company AS";

            // Let the API create the VTFx file 
            if (!file.create(filename, fileSettings))
            {
                return false;
            }

            // Create a single database (could have many)
            // -------------------------------------------------------------------------
            Database singleDatabase = new Database(file, "Single Database", 1); // database ID 1

            // Create a single case (could have many)
            // -------------------------------------------------------------------------
            Case singleCase = new Case(file, "Single Case", 1, 1); // case ID 1 using database with ID 1
            singleCase.Description = "<b>Description</b>";

            // Part 1
            // -------------------------------------------------------------------------
            // Write nodes
            FloatArr nodesPart1 = new FloatArr(NODES_PART_1);
            IntArr nodesPart1Ids = new IntArr(NODES_PART_1_IDS);
            if (!WriteNodes(1, singleDatabase, nodesPart1, nodesPart1Ids))
            {
                return false;
            }

            // Write elements
            IntArr connectsPart1 = new IntArr(CONNECTS_PART_1);
            IntArr elementPart1 = new IntArr(ELEMENTS_PART_1_IDS);
            if (!WriteElements(1, singleDatabase, connectsPart1, elementTypePart1, 1, elementPart1))
            {
                return false; 
            }
    
            // Part 2
            // -------------------------------------------------------------------------
            // Write nodes
            FloatArr nodesPart2 = new FloatArr(NODES_PART_2);
            IntArr nodesPart2Ids = new IntArr(NODES_PART_2_IDS);
            if (!WriteNodes(2, singleDatabase, nodesPart2, nodesPart2Ids))
            {
                return false; 
            }

            // Write elements
            IntArr connectsPart2 = new IntArr(CONNECTS_PART_2);
            IntArr elementPart2 = new IntArr(ELEMENTS_PART_2_IDS);
            if (!WriteElements(2, singleDatabase, connectsPart2, elementTypePart2, 2, elementPart2))
            {
                return false;
            }

            // Write Geometry definition
            // -------------------------------------------------------------------------
            IntArr elementParts = new IntArr(ELEMENT_PARTS);
            if (!WriteGeometry(singleDatabase, elementParts))
            {
                return false;
            }

            // Write the scalar result values
            // -------------------------------------------------------------------------
            // Write result values for each part for each step (here 2 parts, 2 steps)
            FloatArr result1P1S1 = new FloatArr(RESULT1_P1_S1);
            if (!WriteResult(1, singleDatabase, result1P1S1, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.SCALAR1D))
            {
                return false; 
            }

            FloatArr result1P1S2 = new FloatArr(RESULT1_P1_S2);
            if (!WriteResult(11, singleDatabase, result1P1S2, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.SCALAR1D))
            {
                return false; 
            }

            FloatArr result1P2S1 = new FloatArr(RESULT1_P2_S1);
            if (!WriteResult(2, singleDatabase, result1P2S1, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.SCALAR1D))
            {
                return false; 
            }
    
            FloatArr result1P2S2 = new FloatArr(RESULT1_P2_S2);
            if (!WriteResult(12, singleDatabase, result1P2S2, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.SCALAR1D))
            {
                return false; 
            }

            // Write the scalar result definition
            // -------------------------------------------------------------------------
            ResultBlock scalarResult = new ResultBlock(1, ResultBlock.ResultType.SCALAR, ResultBlock.ResultMapping.NODE_MAPPING);
            if (!scalarResult.addResultValuesBlock(1, 1)  || // Assign the result values blocks to state IDs. Order is not significant.
                !scalarResult.addResultValuesBlock(11, 2) ||
                !scalarResult.addResultValuesBlock(2, 1)  ||
                !scalarResult.addResultValuesBlock(12, 2))
            {
                return false;
            }
            scalarResult.ResultId = 1;
            scalarResult.Name = "My scalar result";

            if (!singleDatabase.writeBlock(scalarResult))
            {
                return false; 
            }

            // Write the vector result values
            // -------------------------------------------------------------------------
            // Write result values for each part for each step (here 2 parts, 2 steps)
            FloatArr result2P1S1 = new FloatArr(RESULT2_P1_S1);
            if (!WriteResult(101, singleDatabase, result2P1S1, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result2P1S2 = new FloatArr(RESULT2_P1_S2);
            if (!WriteResult(111, singleDatabase, result2P1S2, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result2P2S1 = new FloatArr(RESULT2_P2_S1);
            if (!WriteResult(102, singleDatabase, result2P2S1, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result2P2S2 = new FloatArr(RESULT2_P2_S2);
            if (!WriteResult(112, singleDatabase, result2P2S2, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            // Write the vector result definition
            // -------------------------------------------------------------------------
            ResultBlock vectorResult = new ResultBlock(2, ResultBlock.ResultType.VECTOR, ResultBlock.ResultMapping.NODE_MAPPING);
            if (!vectorResult.addResultValuesBlock(101, 1) || // Assign the result values blocks to state IDs. Order is not significant.
                !vectorResult.addResultValuesBlock(111, 2) ||
                !vectorResult.addResultValuesBlock(102, 1) ||
                !vectorResult.addResultValuesBlock(112, 2))
            {
                return false;
            }
            vectorResult.ResultId = 1;
            vectorResult.Name = "My vector result";

            if (!singleDatabase.writeBlock(vectorResult))
            {
                return false; 
            }

    
            // Write the Displacement result values
            // -------------------------------------------------------------------------
            // Write result values for each part for each step (here 2 parts, 2 steps)
            FloatArr result3P1S1 = new FloatArr(RESULT3_P1_S1);
            if (!WriteResult(201, singleDatabase, result3P1S1, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result3P1S2 = new FloatArr(RESULT3_P1_S2);
            if (!WriteResult(211, singleDatabase, result3P1S2, 1, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result3P2S1 = new FloatArr(RESULT3_P2_S1);
            if (!WriteResult(202, singleDatabase, result3P2S1, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            FloatArr result3P2S2 = new FloatArr(RESULT3_P2_S2);
            if (!WriteResult(212, singleDatabase, result3P2S2, 2, Block.BlockType.NODES, ResultValuesBlock.ResultValueDimension.VECTOR3D))
            {
                return false;
            }

            // Write the displacement result definition
            // -------------------------------------------------------------------------
            ResultBlock displacementResult = new ResultBlock(3, ResultBlock.ResultType.DISPLACEMENT, ResultBlock.ResultMapping.NODE_MAPPING);
            if (!displacementResult.addResultValuesBlock(201, 1) || // Assign the result values blocks to state IDs. Order is not significant.
                !displacementResult.addResultValuesBlock(211, 2) ||
                !displacementResult.addResultValuesBlock(202, 1) ||
                !displacementResult.addResultValuesBlock(212, 2))
            {
                return false;
            }
            displacementResult.ResultId = 1;
            displacementResult.Name = "My displacement result";

            if (!singleDatabase.writeBlock(displacementResult))
            {
                return false;
            }
    

            // Write state information info (here we have to "steps" of a time series)
            // -------------------------------------------------------------------------
            StateInfoBlock stepInfo = new StateInfoBlock();

            if (!stepInfo.addStateInfo(1, "First step", 1.0f, StateInfoBlock.ReferenceValueType.TIME) ||
                !stepInfo.addStateInfo(2, "Second and last step", 5.0f, StateInfoBlock.ReferenceValueType.TIME))
            {
                return false;
            }

            if (!singleDatabase.writeBlock(stepInfo))
            {
                return false;
            }

            // Create properties
            // -------------------------------------------------------------------------
            if (!CreateProperties(singleCase))
            {
                return false; 
            }

            // Finally close the file
            // -------------------------------------------------------------------------
            if (!file.close())
            {
                return false;
            }

            return true;
        }


        //--------------------------------------------------------------------------------------------------
        // Write the given nodes to the VTFx file
        //--------------------------------------------------------------------------------------------------
        bool WriteNodes(int blockId, Database database, FloatArr nodes, IntArr nodeIds)
        {
            // Create the node block
            NodeBlock nodeBlock = new NodeBlock(blockId, nodes.Count > 0);

            // Set the node data (Note: Ceetron Export expects interleaved node coordinates (xyzxyzxyz...))
            // -------------------------------------------------------------------------
            if (allAtOnce)
            {
                // Alt 1: Supply all nodes at once
                if (!nodeBlock.setNodes(nodes, nodeIds))
                {
                    return false;
                }
            }
            else
            {
                // Alt 2: Add each node individually, using pre-allocation
                if (!nodeBlock.allocateNodes((uint)nodes.Count))
                {
                    return false;
                }

                for (int i = 0; i < nodes.Count/3; i++)
                {
                    if (nodes.Count > 0)
                    {
                        nodeBlock.addNode(nodes[i*3], nodes[i*3 + 1], nodes[i*3 + 2], nodeIds[i]);
                    }
                    else 
                    {
                        nodeBlock.addNode(nodes[i*3], nodes[i*3 + 1], nodes[i*3 + 2]);
                    }
                }
            }

            // Write the block
            // -------------------------------------------------------------------------
            if (!database.writeBlock(nodeBlock))
            {
                return false;
            }
    
            return true;
        }


        //--------------------------------------------------------------------------------------------------
        // Write the given elements to the VTFx file
        //--------------------------------------------------------------------------------------------------
        bool WriteElements(int blockId, Database database, IntArr elementNodes, ElementBlock.ElementType elementType, int nodeBlockId, IntArr elementIds)
        {
            // Create the element block, using index mapping for the nodes
            ElementBlock elementBlock = new ElementBlock(blockId, elementIds.Count > 0, false);

            // Set the element data
            // -------------------------------------------------------------------------
            if (allAtOnce)
            {
                // Set all elements at once (most efficient)
                if (!elementBlock.addElements(elementType, elementNodes, elementIds))
                {
                    return false;
                }
            }
            else
            {
                // Add elements individually (requires frequent re-allocations)

                int numElementNodes = (int)ElementBlock.nodesPerElement(elementType);
                int numElements = elementNodes.Count / numElementNodes; 
                for (int i = 0; i < numElements; i++)
                {
                    IntArr currentElementNodes = elementNodes.GetRange(i* numElementNodes, numElementNodes);
                    if (!elementBlock.addElement(elementType, currentElementNodes, elementIds.Count < 1 ? -1 : elementIds[i]))
                    {
                        return false;
                    }
                }
            }

            elementBlock.NodeBlockId = nodeBlockId;

            // Write the block
            // -------------------------------------------------------------------------
            if (!database.writeBlock(elementBlock))
            {
                return false;
            }
    
            return true;
        }


        //--------------------------------------------------------------------------------------------------
        // Write the geometry definition to the file
        //--------------------------------------------------------------------------------------------------
        bool WriteGeometry(Database database, IntArr elementParts)
        {
            GeometryBlock geoBlock = new GeometryBlock(1); // Only one geometry

            uint geoIndex = 0;

            // Set the element data
            // -------------------------------------------------------------------------
            if (!geoBlock.setElementBlocks(geoIndex, elementParts, elementParts)) // There's one part for each element block. Use same id array for simplicity.
            {
                return false;
            }

            // Write the block
            // -------------------------------------------------------------------------
            if (!database.writeBlock(geoBlock))
            {
                return false;
            }

            // Set info
            // -------------------------------------------------------------------------
            GeometryInfoBlock infoBlock = new GeometryInfoBlock(1); // Only one geometry
            for (int i = 0; i < elementParts.Count; i++)
            {
                infoBlock.addPartInfo(geoIndex, elementParts[i], "Part" + i.ToString()); // Geometry index = 0
            }

            if (!database.writeBlock(infoBlock))
            {
                return false;
            }

            return true;
        }


        //--------------------------------------------------------------------------------------------------
        // Write 1D/3D result values for one part to the file
        //--------------------------------------------------------------------------------------------------
        bool WriteResult(int blockId, Database database, FloatArr resultValues, int mapToBlock, Block.BlockType mapToBlockType, ResultValuesBlock.ResultValueDimension dimension)
        {
            // Create the result values block
            ResultValuesBlock resultValuesBlock = new ResultValuesBlock(blockId, dimension, false);

            resultValuesBlock.setMapToBlockId(mapToBlock, mapToBlockType);

            // Set the result values
            // -------------------------------------------------------------------------
            if (allAtOnce)
            {
                // Alt 1: Set data directly
                if (dimension == ResultValuesBlock.ResultValueDimension.SCALAR1D)
                {
                    if (!resultValuesBlock.setResultValues(resultValues))
                    {
                        return false;
                    }
                }
                else // assuming VECTOR3D
                {
                    // Note: Ceetron Export expects interleaved coordinates for vector results (xyzxyz...)
                    if (!resultValuesBlock.setResultValues(resultValues))
                    {
                        return false;
                    }
                }
            }
            else
            {
                // Alt 2: Add each result value individually (using pre-allocation)
                if (!resultValuesBlock.allocateResultValues(resultValues.Count))
                {
                    return false;
                }

                if (dimension == ResultValuesBlock.ResultValueDimension.SCALAR1D)
                {
                    for (int i = 0; i < resultValues.Count; i++)
                    {
                        resultValuesBlock.addResultValue1d(resultValues[i]);
                    }
                }
                else // assuming VECTOR3D
                {
                    for (int i = 0; i < resultValues.Count / 3; i++)
                    {
                        resultValuesBlock.addResultValue3d(resultValues[3 * i], resultValues[3 * i + 1], resultValues[3 * i + 2]);
                    }
                }
            }

            // Write the block
            // -------------------------------------------------------------------------
            if (!database.writeBlock(resultValuesBlock))
            {
                return false;
            }

            return true;
        }


        //--------------------------------------------------------------------------------------------------
        // Create case properties
        //--------------------------------------------------------------------------------------------------
        bool CreateProperties(Case currentCase)
        {
            PropertySetCollection vtfxProps = new PropertySetCollection();

            PropertySet partSettings = new PropertySet("part_settings");
            partSettings.setValue("context_geometry_index", new Variant((uint)0));
            partSettings.setValue("context_part_id",        new Variant(1));
            partSettings.setValue("draw_style",             new Variant("line"));
            vtfxProps.addPropertySet(partSettings);

            PropertySet scalarSelection = new PropertySet("result_selection");
            scalarSelection.setValue("fringes_result_id", new Variant(1));
            vtfxProps.addPropertySet(scalarSelection);

            PropertySet vectorSelection = new PropertySet("result_selection");
            VariantArr vectorIds = new VariantArr();
            vectorIds.Add(new Variant(1));
            vectorSelection.setValue("vector_result_ids", new Variant(vectorIds));
            vtfxProps.addPropertySet(vectorSelection);

            PropertySet colorMapper = new PropertySet("color_mapper_filled_contours_uniform");
            colorMapper.setValue("context_result_id",   new Variant(1));
            colorMapper.setValue("range_min",           new Variant(1.0));
            colorMapper.setValue("range_max",           new Variant(5.0));
            colorMapper.setValue("color_scheme",        new Variant("thermal_1"));
            vtfxProps.addPropertySet(colorMapper);

            PropertySet vectorSettings = new PropertySet("vector_settings");
            vectorSettings.setValue("context_result_id",    new Variant(1));
            vectorSettings.setValue("color_mode",           new Variant("color_by_fringes"));
            vtfxProps.addPropertySet(vectorSettings);

            // Write the block
            if (!currentCase.setProperties(vtfxProps))
            {
                return false;
            }

            return true;
        }
    }
}
